from contextlib import contextmanager
from shutil import rmtree
import sys
import os
import pytest

from insights.client import InsightsClient
from insights.client.client import get_file_handler, RotatingFileHandlerWithUMask, FileHandlerWithUMask
from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.client.connection import InsightsConnection
from insights import package_info
from insights.client.constants import InsightsConstants as constants
from mock.mock import patch, Mock, call, ANY
from pytest import mark
from pytest import raises

# Temporary directory to mock registration files
TEMP_TEST_REG_DIR = "/tmp/insights-client-registration"
TEMP_TEST_REG_DIR2 = "/tmp/redhat-access-insights-registration"


@pytest.fixture(autouse=True)
def mock_os_chmod():
    with patch('insights.client.client.os.chmod', Mock()) as os_chmod:
        yield os_chmod


@pytest.fixture(autouse=True)
def mock_os_umask():
    with patch('insights.client.client.os.umask', Mock()) as os_umask:
        yield os_umask


class _mock_InsightsConnection(object):
    '''
    For stubbing out network calls
    '''
    def __init__(self, registered=None):
        self.registered = registered

    def api_registration_check(self):
        # True = registered
        # None = unregistered
        # False = unreachable
        # Legacy code:
        # True = system exists in inventory
        # False = Error connection or parsing response
        # None = Machine is not register
        return self.registered

    def register(self):
        return ('msg', 'hostname', "None", "")

    def unregister(self):
        return True


# @TODO DRY the args hack.

def test_version():

    # Hack to prevent client from parsing args to py.test
    tmp = sys.argv
    sys.argv = []

    try:
        config = InsightsConfig(logging_file='/tmp/insights.log')
        client = InsightsClient(config)
        result = client.version()
        assert result == "%s-%s" % (package_info["VERSION"], package_info["RELEASE"])
    finally:
        sys.argv = tmp


@contextmanager
def _mock_no_register_files():
    # mock a directory with the fresh install files
    if not os.path.exists(TEMP_TEST_REG_DIR):
        os.mkdir(TEMP_TEST_REG_DIR)
    if not os.path.exists(TEMP_TEST_REG_DIR2):
        os.mkdir(TEMP_TEST_REG_DIR2)
    try:
        unregistered_path = os.path.join(TEMP_TEST_REG_DIR, ".unregistered")
        with open(unregistered_path, "w") as unregistered_file:
            unregistered_file.write("date")
        unregistered_path2 = os.path.join(TEMP_TEST_REG_DIR2, ".unregistered")
        with open(unregistered_path2, "w") as unregistered_file:
            unregistered_file.write("date")
        yield
    finally:
        rmtree(TEMP_TEST_REG_DIR)
        rmtree(TEMP_TEST_REG_DIR2)


@contextmanager
def _mock_no_register_files_machineid_present():
    # mock a directory with the fresh install files
    if not os.path.exists(TEMP_TEST_REG_DIR):
        os.mkdir(TEMP_TEST_REG_DIR)
    if not os.path.exists(TEMP_TEST_REG_DIR2):
        os.mkdir(TEMP_TEST_REG_DIR2)
    try:
        unregistered_path = os.path.join(TEMP_TEST_REG_DIR, ".unregistered")
        with open(unregistered_path, "w") as unregistered_file:
            unregistered_file.write("date")
        unregistered_path2 = os.path.join(TEMP_TEST_REG_DIR2, ".unregistered")
        with open(unregistered_path2, "w") as unregistered_file:
            unregistered_file.write("date")
        machine_id_path = os.path.join(TEMP_TEST_REG_DIR, "machine-id")
        with open(machine_id_path, "w") as machine_id_file:
            machine_id_file.write("id")
        yield
    finally:
        rmtree(TEMP_TEST_REG_DIR)
        rmtree(TEMP_TEST_REG_DIR2)


@patch('insights.client.client.os.path.dirname')
@patch('insights.client.client.get_version_info')
def test_get_log_handler_by_client_version(get_version_info, mock_path_dirname):
    '''
    Verify that get_log_handler() returns
    the correct file handler depending on
    the client rpm version.
    '''
    mock_path_dirname.return_value = "mock_dirname"

    # RPM version is older than 3.2.0
    get_version_info.return_value = {'client_version': '3.1.8'}
    conf = InsightsConfig(logging_file='/tmp/insights.log')
    assert isinstance(get_file_handler(conf), RotatingFileHandlerWithUMask) is True

    # RPM version is 3.2.0
    get_version_info.return_value = {'client_version': '3.2.0'}
    conf = InsightsConfig(logging_file='/tmp/insights.log')
    assert isinstance(get_file_handler(conf), FileHandlerWithUMask) is True

    # RPM version is newer than 3.2.0
    get_version_info.return_value = {'client_version': '3.2.1'}
    conf = InsightsConfig(logging_file='/tmp/insights.log')
    assert isinstance(get_file_handler(conf), FileHandlerWithUMask) is True

    try:
        os.rmdir(mock_path_dirname.return_value)
    except OSError:
        pass


@patch('insights.client.client.generate_machine_id')
@patch('insights.client.utilities.delete_unregistered_file')
@patch('insights.client.utilities.write_to_disk')
@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_register_legacy(utilities_write, delete_unregistered_file, generate_machine_id):
    config = InsightsConfig(register=True, legacy_upload=True)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=None)
    client.connection.config = config
    client.session = True
    with _mock_no_register_files():
        client.register() is True
    delete_unregistered_file.assert_called_once()
    generate_machine_id.assert_called_once_with()
    utilities_write.assert_has_calls((
        call(constants.registered_files[0]),
        call(constants.registered_files[1])
    ))


@patch('insights.client.client.generate_machine_id')
@patch('insights.client.utilities.delete_unregistered_file')
@patch('insights.client.utilities.write_to_disk')
@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_register_legacy_error_machineid(utilities_write, delete_unregistered_file, generate_machine_id):
    config = InsightsConfig(register=True, legacy_upload=True)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=False)
    client.connection.config = config
    client.session = True
    # this should return False
    with _mock_no_register_files_machineid_present():
        client.register() is False
    delete_unregistered_file.assert_not_called()
    generate_machine_id.assert_not_called()


@contextmanager
def _mock_registered_files():
    # mock a directory with the registered files
    if not os.path.exists(TEMP_TEST_REG_DIR):
        os.mkdir(TEMP_TEST_REG_DIR)
    if not os.path.exists(TEMP_TEST_REG_DIR2):
        os.mkdir(TEMP_TEST_REG_DIR2)
    try:
        registered_path = os.path.join(TEMP_TEST_REG_DIR, ".registered")
        with open(registered_path, "w") as registered_file:
            registered_file.write("date")
        machine_id_path = os.path.join(TEMP_TEST_REG_DIR, "machine-id")
        with open(machine_id_path, "w") as machine_id_file:
            machine_id_file.write("id")
        registered_path2 = os.path.join(TEMP_TEST_REG_DIR2, ".registered")
        with open(registered_path2, "w") as registered_file:
            registered_file.write("date")

        yield
    finally:
        rmtree(TEMP_TEST_REG_DIR)
        rmtree(TEMP_TEST_REG_DIR2)


@patch('insights.client.utilities.write_unregistered_file')
@patch('insights.client.utilities.delete_cache_files')
@patch('insights.client.utilities.write_to_disk')
@patch('insights.client.client.write_to_disk')
@patch('insights.client.utilities.get_time', return_value='now')
@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_unregister_legacy(date, client_write, utilities_write, _delete_cache_file, _write_unregistered_file):
    config = InsightsConfig(unregister=True, legacy_upload=True)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=True)
    client.connection.config = config
    client.session = True
    with _mock_registered_files():
        assert client.unregister() is True
    client_write.assert_called_once_with(constants.machine_id_file, delete=True)
    utilities_write.assert_has_calls((
        call(constants.registered_files[0], delete=True),
        call(constants.registered_files[1], delete=True),
        call(constants.unregistered_files[0], content=date.return_value),
        call(constants.unregistered_files[1], content=date.return_value)
    ))


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
@patch("insights.client.connection.machine_id_exists", return_value=True)
@patch("insights.client.connection.write_unregistered_file")
@patch("insights.client.connection.write_to_disk")
@patch("os.path.exists", side_effect=lambda path: path == constants.registered_files[0])
@patch("insights.client.connection.InsightsConnection._fetch_system_by_machine_id", return_value=False)
def test_unregister_no_inventory(fetch_system_by_machine_id, os_path_exists, write_to_disk, write_unregistered_file, machine_id_exists, get_proxies, _init_session):
    '''
    Test unregister when the system does not exist in Inventory,
    but local registration files exist.
    '''
    config = InsightsConfig(unregister=True, legacy_upload=False)
    connection = InsightsConnection(config)
    result = connection.unregister()

    write_unregistered_file.assert_called_once()
    write_to_disk.assert_called_once_with(constants.machine_id_file, delete=True)
    assert result is True


def test_register_container():
    with pytest.raises(ValueError):
        InsightsConfig(register=True, analyze_container=True)


def test_unregister_container():
    with pytest.raises(ValueError):
        InsightsConfig(unregister=True, analyze_container=True)


@pytest.mark.skip(reason="Mocked paths not working in QE jenkins")
@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_reg_check_registered():

    config = InsightsConfig()
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=True)
    client.connection.config = config
    client.session = True

    with _mock_registered_files():
        assert client.get_registration_status()['status'] is True
        for r in constants.registered_files:
            assert os.path.isfile(r) is True
        for u in constants.unregistered_files:
            assert os.path.isfile(u) is False


@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_reg_check_unregistered():
    # unregister the machine first
    config = InsightsConfig()
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=False)
    client.connection.config = config
    client.session = True

    # test function and integration in .register()
    with _mock_no_register_files():
        assert client.get_registration_status()['status'] is False
        for r in constants.registered_files:
            assert os.path.isfile(r) is False
        for u in constants.unregistered_files:
            assert os.path.isfile(u) is True


@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_reg_check_registered_res_unreg_legacy():
    # system is registered but receives the unregistration status
    config = InsightsConfig(legacy_upload=True)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=None)
    client.connection.config = config
    client.session = True

    # test function and integration in .register()
    with _mock_registered_files():
        assert client.get_registration_status()['status'] is False
        for r in constants.registered_files:
            assert os.path.isfile(r) is False
        for u in constants.unregistered_files:
            assert os.path.isfile(u) is True


@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_reg_check_registered_res_unreg():
    # system is registered but receives the unregistration status
    config = InsightsConfig(legacy_upload=False)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=False)
    client.connection.config = config
    client.session = True

    # test function and integration in .register()
    with _mock_registered_files():
        assert client.get_registration_status() is False
        for r in constants.registered_files:
            assert os.path.isfile(r) is False
        for u in constants.unregistered_files:
            assert os.path.isfile(u) is True


@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_reg_check_registered_unreachable_legacy():
    config = InsightsConfig(legacy_upload=True)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=False)
    client.connection.config = config
    client.session = True

    with _mock_registered_files():
        assert client.get_registration_status()['unreachable'] is True
        for r in constants.registered_files:
            assert os.path.isfile(r) is True
        for u in constants.unregistered_files:
            assert os.path.isfile(u) is False


@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_reg_check_registered_unreachable():
    # If it is unreachable do nothing
    config = InsightsConfig(legacy_upload=False)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=None)
    client.connection.config = config
    client.session = True

    with _mock_registered_files():
        assert client.get_registration_status() is None
        for r in constants.registered_files:
            assert os.path.isfile(r) is True
        for u in constants.unregistered_files:
            assert os.path.isfile(u) is False


@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_reg_check_unregistered_unreachable():
    config = InsightsConfig(legacy_upload=False)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=False)
    client.connection.config = config
    client.session = True

    with _mock_no_register_files():
        assert client.get_registration_status() is False
        for r in constants.registered_files:
            assert os.path.isfile(r) is False
        for u in constants.unregistered_files:
            assert os.path.isfile(u) is True


@patch('insights.client.utilities.constants.registered_files',
       [TEMP_TEST_REG_DIR + '/.registered',
        TEMP_TEST_REG_DIR2 + '/.registered'])
@patch('insights.client.utilities.constants.unregistered_files',
       [TEMP_TEST_REG_DIR + '/.unregistered',
        TEMP_TEST_REG_DIR2 + '/.unregistered'])
@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
def test_reg_check_unregistered_unreachable_legacy():
    config = InsightsConfig(legacy_upload=True)
    client = InsightsClient(config)
    client.connection = _mock_InsightsConnection(registered=False)
    client.connection.config = config
    client.session = True

    with _mock_no_register_files():
        assert client.get_registration_status()['unreachable'] is True
        for r in constants.registered_files:
            assert os.path.isfile(r) is False
        for u in constants.unregistered_files:
            assert os.path.isfile(u) is True


@patch('insights.client.client.constants.sleep_time', 0)
@patch('insights.client.client.InsightsConnection._init_session', return_value=None)
@patch('insights.client.client.InsightsConnection.upload_archive', return_value=Mock(status_code=500))
@patch('insights.client.os.path.exists', return_value=True)
@patch("insights.client.client.logger")
def test_upload_500_retry(logger, _, upload_archive, __):

    # Hack to prevent client from parsing args to py.test
    tmp = sys.argv
    sys.argv = []

    try:
        retries = 3

        config = InsightsConfig(logging_file='/tmp/insights.log', retries=retries)
        client = InsightsClient(config)
        with pytest.raises(RuntimeError):
            client.upload('/tmp/insights.tar.gz')

        upload_archive.assert_called()
        assert upload_archive.call_count == retries
        logger.error.assert_any_call("Upload attempt %d of %d failed! Reason: %s", 1, config.retries, ANY)
        logger.error.assert_called_with("All attempts to upload have failed!")
    finally:
        sys.argv = tmp


@patch('insights.client.client.constants.sleep_time', 0)
@patch('insights.client.client.InsightsConnection._init_session', return_value=None)
@patch('insights.client.client.InsightsConnection.upload_archive')
@patch("insights.client.client.logger")
def test_upload_exception_retry(logger, upload_archive, _):
    from requests.exceptions import ConnectionError, ProxyError, Timeout, HTTPError, SSLError
    upload_archive.side_effect = [ConnectionError("Connection Error"),
                                  ProxyError("Proxy Error"),
                                  Timeout("Timeout Error")]
    retries = 3

    config = InsightsConfig(legacy_upload=False, logging_file='/tmp/insights.log', retries=retries)
    client = InsightsClient(config)
    with patch('insights.client.os.path.exists', return_value=True):
        with pytest.raises(RuntimeError):
            client.upload('/tmp/insights.tar.gz')
    assert upload_archive.call_count == retries
    logger.debug.assert_any_call("Upload attempt %d of %d ...", 1, config.retries)
    logger.error.assert_any_call("Upload attempt %d of %d failed! Reason: %s", 1, config.retries, "Connection Error")
    logger.error.assert_any_call("Upload attempt %d of %d failed! Reason: %s", 2, config.retries, "Proxy Error")
    logger.error.assert_any_call("Upload attempt %d of %d failed! Reason: %s", 3, config.retries, "Timeout Error")
    logger.error.assert_called_with("All attempts to upload have failed!")

    # Test legacy uploads
    logger.reset_mock()
    upload_archive.reset_mock()
    upload_archive.side_effect = [HTTPError("HTTP Error"),
                                  SSLError("SSL Error")]
    retries = 2
    config = InsightsConfig(legacy_upload=True, logging_file='/tmp/insights.log', retries=retries)
    client = InsightsClient(config)
    with patch('insights.client.os.path.exists', return_value=True):
        with pytest.raises(RuntimeError):
            client.upload('/tmp/insights.tar.gz')
    assert upload_archive.call_count == retries
    logger.debug.assert_any_call("Legacy upload attempt %d of %d ...", 1, config.retries)
    logger.error.assert_any_call("Upload attempt %d of %d failed! Reason: %s", 1, config.retries, "HTTP Error")
    logger.error.assert_any_call("Upload attempt %d of %d failed! Reason: %s", 2, config.retries, "SSL Error")
    logger.error.assert_called_with("All attempts to upload have failed!")


@patch('insights.client.client.InsightsConnection._init_session', return_value=None)
@patch('insights.client.client.InsightsConnection.handle_fail_rcs')
@patch('insights.client.client.InsightsConnection.upload_archive', return_value=Mock(status_code=412))
@patch('insights.client.os.path.exists', return_value=True)
def test_upload_412_no_retry(_, upload_archive, handle_fail_rcs, __):

    # Hack to prevent client from parsing args to py.test
    tmp = sys.argv
    sys.argv = []

    try:
        config = InsightsConfig(logging_file='/tmp/insights.log', retries=3)
        client = InsightsClient(config)
        with pytest.raises(RuntimeError):
            client.upload('/tmp/insights.tar.gz')

        upload_archive.assert_called_once()
    finally:
        sys.argv = tmp


@patch('insights.client.client.InsightsConnection._init_session', return_value=None)
@patch('insights.client.connection.write_unregistered_file')
@patch('insights.client.client.InsightsConnection.upload_archive',
       return_value=Mock(**{"status_code": 412,
                            "json.return_value": {"unregistered_at": "now", "message": "msg"}}))
@patch('insights.client.os.path.exists', return_value=True)
def test_upload_412_write_unregistered_file(_, upload_archive, write_unregistered_file, __):

    # Hack to prevent client from parsing args to py.test
    tmp = sys.argv
    sys.argv = []

    try:
        config = InsightsConfig(logging_file='/tmp/insights.log', retries=3)
        client = InsightsClient(config)
        with pytest.raises(RuntimeError):
            client.upload('/tmp/insights.tar.gz')

        unregistered_at = upload_archive.return_value.json()["unregistered_at"]
        write_unregistered_file.assert_called_once_with(unregistered_at)
    finally:
        sys.argv = tmp


@patch('insights.client.archive.InsightsArchive.storing_archive')
@patch('insights.client.archive.tempfile.mkdtemp', Mock())
@patch('insights.client.archive.os.makedirs', Mock())
@patch('insights.client.archive.atexit', Mock())
def test_cleanup_tmp(storing_archive):
    config = InsightsConfig(keep_archive=False)
    arch = InsightsArchive(config)
    arch.tmp_dir = "/test"
    arch.tar_file = "/test/test.tar.gz"
    arch.keep_archive_dir = "/test-keep-archive"
    arch.cleanup_tmp()
    assert not os.path.exists(arch.tmp_dir)
    storing_archive.assert_not_called()

    config.keep_archive = True
    arch.cleanup_tmp()
    assert not os.path.exists(arch.tmp_dir)
    storing_archive.assert_called_once()


@patch('insights.client.archive.InsightsArchive.storing_archive')
@patch('insights.client.archive.tempfile.mkdtemp', Mock())
@patch('insights.client.archive.os.makedirs', Mock())
@patch('insights.client.archive.atexit', Mock())
def test_cleanup_tmp_obfuscation(storing_archive):
    config = InsightsConfig(keep_archive=False, obfuscate=True)
    arch = InsightsArchive(config)
    arch.tmp_dir = '/var/tmp/insights-archive-000000'
    arch.tar_file = '/var/tmp/insights-archive-test.tar.gz'
    arch.keep_archive_dir = '/var/tmp/test-archive'
    arch.cleanup_tmp()
    assert not os.path.exists(arch.tmp_dir)
    storing_archive.assert_not_called()

    arch.config.keep_archive = True
    arch.cleanup_tmp()
    assert not os.path.exists(arch.tmp_dir)
    storing_archive.assert_called_once()


@patch('insights.client.client._legacy_handle_registration')
def test_legacy_register(_legacy_handle_registration):
    '''
    _legacy_handle_registration called when legacy upload
    '''
    config = InsightsConfig(legacy_upload=True)
    client = InsightsClient(config)
    client.register()
    _legacy_handle_registration.assert_called_once()


@patch('insights.client.client._legacy_handle_unregistration')
def test_legacy_unregister(_legacy_handle_unregistration):
    '''
    _legacy_handle_unregistration called when legacy upload
    '''
    config = InsightsConfig(legacy_upload=True)
    client = InsightsClient(config)
    client.unregister()
    _legacy_handle_unregistration.assert_called_once()


@patch('insights.client.client.handle_registration')
def test_register_upload(handle_registration):
    '''
    handle_registration called when upload
    '''
    config = InsightsConfig(legacy_upload=False)
    client = InsightsClient(config)
    client.register()
    handle_registration.assert_called_once()


@patch('insights.client.client.handle_unregistration')
def test_unregister_upload(handle_unregistration):
    '''
    handle_unregistration called when upload
    '''
    config = InsightsConfig(legacy_upload=False)
    client = InsightsClient(config)
    client.unregister()
    handle_unregistration.assert_called_once()


@patch('insights.client.os.path.exists', return_value=True)
@patch('insights.client.connection.InsightsConnection.upload_archive', Mock(return_value=Mock(status_code=200)))
@patch('insights.client.client._legacy_upload')
@patch('insights.client.client.write_to_disk', Mock())
def test_legacy_upload(_legacy_upload, path_exists):
    '''
    _legacy_upload called when legacy upload
    '''
    config = InsightsConfig(legacy_upload=True)
    client = InsightsClient(config)
    client.upload('test.gar.gz', 'test.content.type')
    _legacy_upload.assert_called_once()


@patch('insights.client.os.path.exists', return_value=True)
@patch('insights.client.connection.InsightsConnection.upload_archive', Mock(return_value=Mock(status_code=200)))
@patch('insights.client.client._legacy_upload')
@patch('insights.client.client.write_to_disk', Mock())
def test_platform_upload(_legacy_upload, path_exists):
    '''
    _legacy_upload not called when platform upload
    '''
    config = InsightsConfig(legacy_upload=False)
    client = InsightsClient(config)
    client.upload('test.gar.gz', 'test.content.type')
    _legacy_upload.assert_not_called()


@patch('insights.client.os.path.exists', return_value=True)
@patch('insights.client.connection.InsightsConnection.upload_archive', return_value=Mock(status_code=200))
@patch('insights.client.client._legacy_upload')
def test_platform_upload_with_no_log_path(_legacy_upload, _, path_exists):
    '''
    testing logger when no path is given
    '''
    config = InsightsConfig(legacy_upload=True, logging_file='tmp.log')
    client = InsightsClient(config)
    response = client.upload('test.gar.gz', 'test.content.type')
    _legacy_upload.assert_called_once()
    assert response is not None


@patch('insights.client.shutil')
def test_copy_to_output_dir(shutil_):
    '''
    Test that shutil is called to copy the collection to
    the specified output dir
    '''
    config = InsightsConfig()
    client = InsightsClient(config)
    client.copy_to_output_dir('test')
    shutil_.copytree.assert_called_once()


@patch('insights.client.shutil')
@patch('insights.client.os')
def test_copy_to_output_dir_exists_and_empty(os_, shutil_):
    '''
    Test that writing to an existing but empty directory is
    performed
    '''
    config = InsightsConfig(output_dir='dest')
    client = InsightsClient(config)
    # raise file exists error first, then raise for "b" and "c" below
    shutil_.copytree.side_effect = [OSError(17, 'File exists'), None, None]

    # returns empty list for destination, file list for source
    os_.listdir.side_effect = [[], ['a', 'b', 'c']]

    # os.path.join called 6 times, once for each file per src and dest
    os_.path.join.side_effect = [
        os.path.join('src', 'a'),
        os.path.join(config.output_dir, 'a'),
        os.path.join('src', 'b'),
        os.path.join(config.output_dir, 'b'),
        os.path.join('src', 'c'),
        os.path.join(config.output_dir, 'c')]

    # 'a' is file, 'b', 'c' are dirs
    os_.path.isfile.side_effect = [True, False, False]
    # a is file so the check for 'a' does not fall through to the elif
    os_.path.isdir.side_effect = [True, True]

    client.copy_to_output_dir('src')

    os_.listdir.assert_has_calls([call(config.output_dir), call('src')])
    os_.path.isfile.assert_has_calls([call('src/a'), call('src/b'), call('src/c')])
    # a is file so the check for 'a' does not fall through to the elif
    os_.path.isdir.assert_has_calls([call('src/b'), call('src/c')])
    # initial (failed) copy is part of the calls
    shutil_.copytree.assert_has_calls([
        call('src', config.output_dir),
        call(os.path.join('src', 'b'), os.path.join(config.output_dir, 'b')),
        call(os.path.join('src', 'c'), os.path.join(config.output_dir, 'c'))])
    shutil_.copyfile.assert_has_calls([
        call(os.path.join('src', 'a'), os.path.join(config.output_dir, 'a'))])


@patch('insights.client.shutil')
@patch('insights.client.os')
def test_copy_to_output_dir_exists_and_empty_err_during_copy(os_, shutil_):
    '''
    Test that when writing to an existing but empty directory,
    if an error occurs, we bail out before finishing.
    '''
    config = InsightsConfig(output_dir='dest')
    client = InsightsClient(config)
    # raise file exists error first, then raise nothing for "b" and "c" below
    shutil_.copytree.side_effect = [OSError(17, 'File exists'), None, None]

    # raise an unknown error for "a"
    shutil_.copyfile.side_effect = [OSError(19, '???'), None, None]

    # returns empty list for destination, file list for source
    os_.listdir.side_effect = [[], ['a', 'b', 'c']]

    # os.path.join called 6 times, once for each file per src and dest
    os_.path.join.side_effect = [
        os.path.join('src', 'a'),
        os.path.join(config.output_dir, 'a'),
        os.path.join('src', 'b'),
        os.path.join(config.output_dir, 'b'),
        os.path.join('src', 'c'),
        os.path.join(config.output_dir, 'c')]

    # 'a' is file, 'b', 'c' are dirs
    os_.path.isfile.side_effect = [True, False, False]
    # a is file so the check for 'a' does not fall through to the elif
    os_.path.isdir.side_effect = [True, True]

    client.copy_to_output_dir('src')

    os_.listdir.assert_has_calls([call(config.output_dir), call('src')])
    os_.path.isfile.assert_has_calls([call('src/a')])
    # a is file so the check for 'a' does not fall through to the elif
    os_.path.isdir.assert_not_called()
    # initial (failed) copy is part of the calls
    shutil_.copytree.assert_has_calls([call('src', config.output_dir)])
    shutil_.copyfile.assert_has_calls([
        call(os.path.join('src', 'a'), os.path.join(config.output_dir, 'a'))])


@patch('insights.client.shutil')
@patch('insights.client.os')
def test_copy_to_output_dir_exists_and_not_empty(os_, shutil_):
    '''
    Test that writing to an existing and non-empty directory is
    NOT performed. Due to the check in config.py this should never happen,
    but just to be safe.
    '''
    config = InsightsConfig(output_dir='dest')
    client = InsightsClient(config)
    shutil_.copytree.side_effect = [OSError(17, 'File exists')]

    os_.listdir.return_value = ['test']

    client.copy_to_output_dir('src')

    os_.listdir.assert_called_once_with(config.output_dir)
    shutil_.copytree.assert_called_once_with('src', config.output_dir)
    shutil_.copyfile.assert_not_called()


@patch('insights.client.shutil')
@patch('insights.client.os')
def test_copy_to_output_dir_other_oserror(os_, shutil_):
    '''
    Test that any OSError != 17 is logged and we bail out
    before attempting to copy anything else
    '''
    config = InsightsConfig(output_dir='dest')
    client = InsightsClient(config)
    shutil_.copytree.side_effect = [OSError(19, '???'), None, None]

    client.copy_to_output_dir('src')

    os_.listdir.assert_not_called
    shutil_.copytree.assert_called_once_with('src', config.output_dir)
    shutil_.copyfile.assert_not_called()


@patch('insights.client.shutil')
def test_copy_to_output_file(shutil_):
    '''
    Test that shutil is called to copy the collection to
    the specified output file
    '''
    config = InsightsConfig()
    client = InsightsClient(config)
    client.copy_to_output_file('test')
    shutil_.copyfile.assert_called_once()


@patch('insights.client.shutil')
def test_copy_to_output_file_obfuscate_on(shutil_):
    '''
    Test that shutil is called to copy the collection to
    the specified output file, and soscleaner copy function
    is called
    '''
    # obfuscate off, no soscleaner files
    config = InsightsConfig(obfuscate=True)
    client = InsightsClient(config)
    client.copy_to_output_file('test')
    shutil_.copyfile.assert_called_once()


@mark.parametrize(("expected_result",), ((True,), (None,)))
def test_checkin_result(expected_result):
    config = InsightsConfig()
    client = InsightsClient(config)
    client.connection = Mock(**{"checkin.return_value": expected_result})
    client.session = True

    actual_result = client.checkin()
    client.connection.checkin.assert_called_once_with()
    assert actual_result is expected_result


def test_checkin_error():
    config = InsightsConfig()
    client = InsightsClient(config)
    client.connection = Mock(**{"checkin.side_effect": Exception})
    client.session = True

    with raises(Exception):
        client.checkin()

    client.connection.checkin.assert_called_once_with()


def test_checkin_offline():
    config = InsightsConfig(offline=True)
    client = InsightsClient(config)
    client.connection = Mock()

    result = client.checkin()
    assert result is None
    client.connection.checkin.assert_not_called()



import doctest

from insights.parsers import crio_conf
from insights.parsers.crio_conf import CrioConf
from insights.tests import context_wrap

CRIO_CONF_DATA = """
[crio]
storage_option=[ "overlay.imagestore=/mnt/overlay", "overlay.size=1G", ]

[crio.runtime]
hooks_dir = [ "/etc/containers/oci/hooks.d", ]
selinux = true
default_sysctls = [
]

test_sysctls = [
\t
]

default_env = [
    "NSS_SDB_USE_CACHE=no",
]

[crio.network]
plugin_dirs = [
    "/var/lib/cni/bin",
    "/usr/libexec/cni",
]

[crio.metrics]
metrics_opt = [ "operations" ]
enable_metrics = true
metrics_port = 9537
metrics_collectors = [
  "operations",
  "operations_latency_microseconds_total",
  "operations_latency_microseconds",
  "operations_errors"
  "containers_oom_total",
  "containers_oom",
  # Drop metrics with excessive label cardinality.
  # "image_pulls_by_digest",
  "image_pulls_layer_size",
  # "image_pulls_by_name",
  # "image_pulls_by_name_skipped",
  # "image_pulls_failures",
  # "image_pulls_successes",
  # "image_layer_reuse",
]

[crio.runtime.workloads]
""".strip()

CONF_PATH = '/etc/crio/crio.conf'


def test_crio_conf():
    confs = CrioConf(context_wrap(CRIO_CONF_DATA, path=CONF_PATH))
    assert 'crio' in confs.sections()
    assert 'crio.runtime.workloads' in confs.sections()
    assert confs.file_name == 'crio.conf'
    assert confs.has_option('crio', 'storage_option')
    assert confs.has_option('crio.runtime', 'hooks_dir')
    assert confs.has_option('crio.metrics', 'metrics_opt')
    assert confs.has_option('crio.metrics', 'metrics_collectors')
    assert confs.has_option('crio.runtime', 'default_sysctls')
    assert confs.has_option('crio.runtime', 'test_sysctls')
    assert confs.has_option('crio.runtime', 'default_env')
    assert "NSS_SDB_USE_CACHE=no" in confs.get('crio.runtime', 'default_env')
    assert confs.has_option('crio.network', 'plugin_dirs')
    assert "/var/lib/cni/bin" in confs.get('crio.network', 'plugin_dirs')
    assert "/usr/libexec/cni" in confs.get('crio.network', 'plugin_dirs')
    assert confs.get('crio.metrics', 'metrics_port') == '9537'


def test_crio_conf_documentation():
    failed_count, tests = doctest.testmod(
        crio_conf,
        globs={'crio_conf': CrioConf(context_wrap(CRIO_CONF_DATA))}
    )
    assert failed_count == 0



import doctest

from insights.parsers import neutron_sriov_agent
from insights.tests import context_wrap

NEUTRON_SRIOV_AGENT_CONF = """
[DEFAULT]
debug = false
verbose = false

[sriov_nic]
physical_device_mappings=datacentre:enp2s0f6

[agent]
polling_interval=2

[securitygroup]
firewall_driver=noop
report_interval = 60

[keystone_authtoken]
"""


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        neutron_sriov_agent,
        globs={'conf': neutron_sriov_agent.NeutronSriovAgent(context_wrap(NEUTRON_SRIOV_AGENT_CONF))}
    )
    assert failed_count == 0


def test_neutron_sriov_agent():
    n_sriov_agent = neutron_sriov_agent.NeutronSriovAgent(context_wrap(NEUTRON_SRIOV_AGENT_CONF))
    assert n_sriov_agent is not None
    assert list(n_sriov_agent.sections()) == [
        'sriov_nic', 'agent', 'securitygroup', 'keystone_authtoken']
    assert n_sriov_agent.defaults() == {
        'debug': 'false',
        'verbose': 'false'}
    assert n_sriov_agent.get('sriov_nic', 'physical_device_mappings') == 'datacentre:enp2s0f6'
    assert n_sriov_agent.has_option('securitygroup', 'firewall_driver')
    assert not n_sriov_agent.has_option('yabba', 'dabba_do')
    assert n_sriov_agent.get('DEFAULT', 'debug') == 'false'



import doctest
from datetime import datetime

from collections import defaultdict

from insights.core import filters
from insights.parsers import rhsm_log
from insights.parsers.rhsm_log import RhsmLog
from insights.specs import Specs
from insights.tests import context_wrap

LOG1 = """
2016-07-31 04:06:41,215 [DEBUG] rhsmcertd-worker:24440 @identity.py:131 - Loading consumer info from identity certificates.
2016-07-31 04:06:41,221 [DEBUG] rhsmcertd-worker:24440 @connection.py:475 - Loaded CA certificates from /etc/rhsm/ca/: redhat-uep.pem
2016-07-31 04:06:41,221 [DEBUG] rhsmcertd-worker:24440 @connection.py:523 - Making request: GET /subscription/consumers/a808d48e-36bf-4071-a00a-0efacc511b2b/certificates/serials
2016-07-31 04:07:21,245 [ERROR] rhsmcertd-worker:24440 @entcertlib.py:121 - [Errno -2] Name or service not known
""".strip()

LOG2 = """
2011-12-27 08:41:12,460 [DEBUG]  @connection.py:209 - Making request: GET /subscription/users/isavia_sysdep/owners
2011-12-27 08:41:13,104 [ERROR]  @managercli.py:65 - Error during registration: certificate verify failed
2011-12-27 08:41:13,104 [ERROR]  @managercli.py:66 - certificate verify failed
    Traceback (most recent call last):
    File "/usr/share/rhsm/subscription_manager/managercli.py", line 600, in _do_command
""".strip()

# For Coverage
LOG3 = """
[ERROR]
2011-12-27-08:41:13,104 [ERROR]  @managercli.py:66 - certificate verify failed
"""

filters.add_filter(Specs.rhsm_log, ["[ERROR]", "[Errno"])


def teardown_function(func):
    filters._CACHE = {}
    filters.FILTERS = defaultdict(dict)


def test_rhsm_log():
    rlog = RhsmLog(context_wrap(LOG1))
    ern_list = rlog.get('[Errno -2]')
    assert 1 == len(ern_list)
    assert (
        ern_list[0]['raw_message']
        == "2016-07-31 04:07:21,245 [ERROR] rhsmcertd-worker:24440 @entcertlib.py:121 - [Errno -2] Name or service not known"
    )
    assert ern_list[0]['timestamp'] == datetime(2016, 7, 31, 4, 7, 21, 245000)
    assert (
        ern_list[0]['message']
        == "[ERROR] rhsmcertd-worker:24440 @entcertlib.py:121 - [Errno -2] Name or service not known"
    )

    rlog = RhsmLog(context_wrap(LOG2))
    ern_list = rlog.get('[Errno -2]')
    assert 0 == len(ern_list)
    err_list = rlog.get('ERROR')
    assert 2 == len(err_list)

    rlog = RhsmLog(context_wrap(LOG3))
    err_list = rlog.get('ERROR')
    assert err_list[0].get('timestamp') is None
    assert err_list[1].get('timestamp') is None


def test_doc():
    failed_count, tests = doctest.testmod(rhsm_log, globs={'rhsm_log': RhsmLog(context_wrap(LOG1))})
    assert failed_count == 0



import doctest

from insights.parsers import files_dirs_number_of_dirs
from insights.tests import context_wrap


OUTPUT = """
{
  "/var/spool/postfix/maildrop/": {"dirs_number": 1, "files_number": 5},
  "/var/spool/clientmqueue/": {"dirs_number": 1, "files_number": 2}
}
""".strip()


def test_files_number_of_dir():
    output = files_dirs_number_of_dirs.FilesDirsNumberOfDir(context_wrap(OUTPUT))
    assert output.data["/var/spool/postfix/maildrop/"]["files_number"] == 5
    assert output.data["/var/spool/clientmqueue/"]["dirs_number"] == 1
    assert output.files_number_of("/var/spool/postfix/maildrop/") == 5
    assert output.dirs_number_of("/var/spool/clientmqueue/") == 1
    assert output.dirs_number_of("/var/spool/clientmqueue") == 1
    # no such dir
    assert output.dirs_number_of("/var/spool/clienttest/") is None
    assert output.files_number_of("/var/spool/clienttest/") is None


def test_doc_examples():
    env = {'filesnumberofdir': files_dirs_number_of_dirs.FilesDirsNumberOfDir(context_wrap(OUTPUT))}
    failed, total = doctest.testmod(files_dirs_number_of_dirs, globs=env)
    assert failed == 0



from collections import namedtuple

import pytest
from insights.parsers.lvm import (Lvs, LvsHeadings, Pvs, PvsHeadings, Vgs,
                                  VgsHeadings)
from insights.parsers.lvm import (LvsAll, PvsAll, VgsAll)
from insights.combiners.lvm import Lvm, LvmAll
from insights.combiners import lvm
from insights.tests import context_wrap


LVS_HEADINGS = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
  LV            VG              Attr       LSize   Pool  Origin Data%  Meta%  Move Log Cpy%Sync Convert LV Tags Devices
  lv_brick1     data1           Vwi-aotz--   3.62t pool1        0.05
  lv_hdfs1      data1           Vwi-aotz--   3.62t pool1        0.05
  pool1         data1           twi-aotz--   3.62t              0.10   0.11                                     pool1_tdata(0)
  [pool1_tdata] data1           Twi-ao----   3.62t                                                              /dev/sdb1(4096)
  [pool1_tmeta] data1           ewi-ao----  16.00g                                                              /dev/sdb1(0)
  lv_brick2     data2           Vwi-aotz--   3.62t pool2        0.05
  lv_hdfs2      data2           Vwi-aotz--   3.62t pool2        0.05
  pool2         data2           twi-aotz--   3.62t              0.10   0.11                                     pool2_tdata(0)
  [pool2_tdata] data2           Twi-ao----   3.62t                                                              /dev/sdc1(4096)
  [pool2_tmeta] data2           ewi-ao----  16.00g                                                              /dev/sdc1(0)
  lv_brick3     data3           Vwi-aotz--   3.62t pool3        0.05
  lv_hdfs3      data3           Vwi-aotz--   3.62t pool3        0.05
  pool3         data3           twi-aotz--   3.62t              0.10   0.11                                     pool3_tdata(0)
  [pool3_tdata] data3           Twi-ao----   3.62t                                                              /dev/sdd1(4096)
  [pool3_tmeta] data3           ewi-ao----  16.00g                                                              /dev/sdd1(0)
  lv_brick4     data4           Vwi-aotz--   3.62t pool4        0.05
  lv_hdfs4      data4           Vwi-aotz--   3.62t pool4        0.05
  pool4         data4           twi-aotz--   3.62t              0.10   0.11                                     pool4_tdata(0)
  [pool4_tdata] data4           Twi-ao----   3.62t                                                              /dev/sde1(4096)
  [pool4_tmeta] data4           ewi-ao----  16.00g                                                              /dev/sde1(0)
  lv_brick5     data5           Vwi-aotz--   3.62t pool5        0.05
  lv_hdfs5      data5           Vwi-aotz--   3.62t pool5        0.05
  pool5         data5           twi-aotz--   3.62t              0.10   0.11                                     pool5_tdata(0)
  [pool5_tdata] data5           Twi-ao----   3.62t                                                              /dev/sdf1(4096)  
  [pool5_tmeta] data5           ewi-ao----  16.00g                                                              /dev/sdf1(0)     
  lv_brick6     data6           Vwi-aotz--   3.62t pool6        0.05                                                             
  lv_hdfs6      data6           Vwi-aotz--   3.62t pool6        0.05                                                             
  pool6         data6           twi-aotz--   3.62t              0.10   0.11                                     pool6_tdata(0)   
  [pool6_tdata] data6           Twi-ao----   3.62t                                                              /dev/sdg1(4096)  
  [pool6_tmeta] data6           ewi-ao----  16.00g                                                              /dev/sdg1(0)     
  lv_brick7     data7           Vwi-aotz--   3.62t pool7        0.05                                                             
  lv_hdfs7      data7           Vwi-aotz--   3.62t pool7        0.05                                                             
  pool7         data7           twi-aotz--   3.62t              0.10   0.11                                     pool7_tdata(0)   
  [pool7_tdata] data7           Twi-ao----   3.62t                                                              /dev/sdh1(4096)  
  [pool7_tmeta] data7           ewi-ao----  16.00g                                                              /dev/sdh1(0)     
  home          rhel_ceehadoop1 -wi-ao---- 976.00g                                                              /dev/sda3(12800) 
  opt           rhel_ceehadoop1 -wi-ao----   2.57t                                                              /dev/sda3(279551)
  root          rhel_ceehadoop1 -wi-ao----  50.00g                                                              /dev/sda3(0)     
  swap          rhel_ceehadoop1 -wi-ao----  16.00g                                                              /dev/sda3(262656)
  var           rhel_ceehadoop1 -wi-ao----  50.00g                                                              /dev/sda3(266752)
""".strip()   # noqa: W291

LVS_NO_HEADINGS = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
  LVM2_LV_NAME='lv_brick1'|LVM2_VG_NAME='data1'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_hdfs1'|LVM2_VG_NAME='data1'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='pool1'|LVM2_VG_NAME='data1'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='twi-aotz--'|LVM2_DEVICES='pool1_tdata(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool1_tdata]'|LVM2_VG_NAME='data1'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Twi-ao----'|LVM2_DEVICES='/dev/sdb1(4096)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool1_tmeta]'|LVM2_VG_NAME='data1'|LVM2_LV_SIZE='16.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='ewi-ao----'|LVM2_DEVICES='/dev/sdb1(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_brick2'|LVM2_VG_NAME='data2'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_hdfs2'|LVM2_VG_NAME='data2'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='pool2'|LVM2_VG_NAME='data2'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='twi-aotz--'|LVM2_DEVICES='pool2_tdata(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool2_tdata]'|LVM2_VG_NAME='data2'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Twi-ao----'|LVM2_DEVICES='/dev/sdc1(4096)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool2_tmeta]'|LVM2_VG_NAME='data2'|LVM2_LV_SIZE='16.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='ewi-ao----'|LVM2_DEVICES='/dev/sdc1(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_brick3'|LVM2_VG_NAME='data3'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_hdfs3'|LVM2_VG_NAME='data3'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='pool3'|LVM2_VG_NAME='data3'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='twi-aotz--'|LVM2_DEVICES='pool3_tdata(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool3_tdata]'|LVM2_VG_NAME='data3'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Twi-ao----'|LVM2_DEVICES='/dev/sdd1(4096)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool3_tmeta]'|LVM2_VG_NAME='data3'|LVM2_LV_SIZE='16.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='ewi-ao----'|LVM2_DEVICES='/dev/sdd1(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_brick4'|LVM2_VG_NAME='data4'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_hdfs4'|LVM2_VG_NAME='data4'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='pool4'|LVM2_VG_NAME='data4'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='twi-aotz--'|LVM2_DEVICES='pool4_tdata(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool4_tdata]'|LVM2_VG_NAME='data4'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Twi-ao----'|LVM2_DEVICES='/dev/sde1(4096)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool4_tmeta]'|LVM2_VG_NAME='data4'|LVM2_LV_SIZE='16.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='ewi-ao----'|LVM2_DEVICES='/dev/sde1(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_brick5'|LVM2_VG_NAME='data5'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_hdfs5'|LVM2_VG_NAME='data5'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='pool5'|LVM2_VG_NAME='data5'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='twi-aotz--'|LVM2_DEVICES='pool5_tdata(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool5_tdata]'|LVM2_VG_NAME='data5'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Twi-ao----'|LVM2_DEVICES='/dev/sdf1(4096)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool5_tmeta]'|LVM2_VG_NAME='data5'|LVM2_LV_SIZE='16.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='ewi-ao----'|LVM2_DEVICES='/dev/sdf1(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_brick6'|LVM2_VG_NAME='data6'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_hdfs6'|LVM2_VG_NAME='data6'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='pool6'|LVM2_VG_NAME='data6'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='twi-aotz--'|LVM2_DEVICES='pool6_tdata(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool6_tdata]'|LVM2_VG_NAME='data6'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Twi-ao----'|LVM2_DEVICES='/dev/sdg1(4096)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool6_tmeta]'|LVM2_VG_NAME='data6'|LVM2_LV_SIZE='16.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='ewi-ao----'|LVM2_DEVICES='/dev/sdg1(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_brick7'|LVM2_VG_NAME='data7'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='lv_hdfs7'|LVM2_VG_NAME='data7'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Vwi-aotz--'|LVM2_DEVICES=''|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='pool7'|LVM2_VG_NAME='data7'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='twi-aotz--'|LVM2_DEVICES='pool7_tdata(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool7_tdata]'|LVM2_VG_NAME='data7'|LVM2_LV_SIZE='3.62t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='Twi-ao----'|LVM2_DEVICES='/dev/sdh1(4096)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='[pool7_tmeta]'|LVM2_VG_NAME='data7'|LVM2_LV_SIZE='16.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='ewi-ao----'|LVM2_DEVICES='/dev/sdh1(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='home'|LVM2_VG_NAME='rhel_ceehadoop1'|LVM2_LV_SIZE='976.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='-wi-ao----'|LVM2_DEVICES='/dev/sda3(12800)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='opt'|LVM2_VG_NAME='rhel_ceehadoop1'|LVM2_LV_SIZE='2.57t'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='-wi-ao----'|LVM2_DEVICES='/dev/sda3(279551)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='root'|LVM2_VG_NAME='rhel_ceehadoop1'|LVM2_LV_SIZE='50.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='-wi-ao----'|LVM2_DEVICES='/dev/sda3(0)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='swap'|LVM2_VG_NAME='rhel_ceehadoop1'|LVM2_LV_SIZE='16.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='-wi-ao----'|LVM2_DEVICES='/dev/sda3(262656)'|LVM2_REGION_SIZE='0 '
  LVM2_LV_NAME='var'|LVM2_VG_NAME='rhel_ceehadoop1'|LVM2_LV_SIZE='50.00g'|LVM2_REGION_SIZE='0 '|LVM2_MIRROR_LOG=''|LVM2_LV_ATTR='-wi-ao----'|LVM2_DEVICES='/dev/sda3(266752)'|LVM2_REGION_SIZE='0 '
""".strip()   # noqa: W291

LVS_NOHEADINGS_POOL7 = {
    'LV': 'pool7',
    'VG': 'data7',
    'LSize': '3.62t',
    'Region': '0',
    'Attr': 'twi-aotz--',
    'Log': None,
    'Devices': 'pool7_tdata(0)'
}

LVS_HEADINGS_POOL7 = {
    'LV': 'pool7',
    'VG': 'data7',
    'Attr': 'twi-aotz--',
    'LSize': '3.62t',
    'Pool': None,
    'Origin': None,
    'Data%': '0.10',
    'Meta%': '0.11',
    'Move': None,
    'Log': None,
    'Cpy%Sync': None,
    'Convert': None,
    'LV_Tags': None,
    'Devices': 'pool7_tdata(0)'
}

PVS_HEADINGS = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
    Wiping internal VG cache
    Wiping cache of LVM-capable devices
  PV                                      VG              Fmt  Attr PSize PFree DevSize PV UUID                                PMdaFree  PMdaSize  #PMda #PMdaUse PE    
  /dev/data1/lv_brick1                                         ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data1/lv_hdfs1                                          ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data2/lv_brick2                                         ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data2/lv_hdfs2                                          ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data3/lv_brick3                                         ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data3/lv_hdfs3                                          ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data4/lv_brick4                                         ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data4/lv_hdfs4                                          ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data5/lv_brick5                                         ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data5/lv_hdfs5                                          ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data6/lv_brick6                                         ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data6/lv_hdfs6                                          ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data7/lv_brick7                                         ---     0     0    3.62t                                               0         0      0        0      0
  /dev/data7/lv_hdfs7                                          ---     0     0    3.62t                                               0         0      0        0      0
  /dev/loop0                                                   ---     0     0  100.00g                                               0         0      0        0      0
  /dev/loop1                                                   ---     0     0    2.00g                                               0         0      0        0      0
  /dev/mapper/docker-253:3-100685290-pool                      ---     0     0  100.00g                                               0         0      0        0      0
  /dev/rhel_ceehadoop1/home                                    ---     0     0  976.00g                                               0         0      0        0      0
  /dev/rhel_ceehadoop1/opt                                     ---     0     0    2.57t                                               0         0      0        0      0
  /dev/rhel_ceehadoop1/root                                    ---     0     0   50.00g                                               0         0      0        0      0
  /dev/rhel_ceehadoop1/swap                                    ---     0     0   16.00g                                               0         0      0        0      0
  /dev/rhel_ceehadoop1/var                                     ---     0     0   50.00g                                               0         0      0        0      0
  /dev/sda2                                                    ---     0     0    1.00g                                               0         0      0        0      0
  /dev/sda3                               rhel_ceehadoop1 lvm2 a--  3.63t    0    3.63t 3e2mRe-1y3a-iUSj-F3tH-yG1M-t3qD-8uNDf5        0   1020.00k     1        1 952831
  /dev/sdb1                               data1           lvm2 a--  3.64t 4.00m   3.64t dhr134-VxSA-R7dW-op7s-HCco-7J2A-7MvaeN        0    252.00k     1        1 953853
  /dev/sdc1                               data2           lvm2 a--  3.64t 4.00m   3.64t sxTsQE-raAM-8s3b-FSy0-LhL2-Naue-f2ebcA        0    252.00k     1        1 953853
  /dev/sdd1                               data3           lvm2 a--  3.64t 4.00m   3.64t yxj2qs-qOKA-57XF-228r-qWQj-3zez-i0Vaiw        0    252.00k     1        1 953853
  /dev/sde1                               data4           lvm2 a--  3.64t 4.00m   3.64t 94oczR-Utni-dOzj-3CMQ-efOJ-TVa8-yak25P        0    252.00k     1        1 953853
  /dev/sdf1                               data5           lvm2 a--  3.64t 4.00m   3.64t r3UW8H-pLz5-rpsn-GK2u-1kJE-GKsy-uOeoIz        0    252.00k     1        1 953853
  /dev/sdg1                               data6           lvm2 a--  3.64t 4.00m   3.64t oZtJSv-HM9w-4She-1TU0-Z31G-dgKX-liJNID        0    252.00k     1        1 953853
  /dev/sdh1                               data7           lvm2 a--  3.64t 4.00m   3.64t Ys0p6S-WJOW-TAZT-czmX-iOYq-0Tgg-v4cl0H        0    252.00k     1        1 953853
    Reloading config files
    Wiping internal VG cache
""".strip()   # noqa: W291

PVS_NO_HEADINGS = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data1/lv_brick1'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='28'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data1/lv_hdfs1'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='41'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data2/lv_brick2'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='30'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data2/lv_hdfs2'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='42'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data3/lv_brick3'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='32'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data3/lv_hdfs3'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='43'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data4/lv_brick4'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='34'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data4/lv_hdfs4'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='44'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data5/lv_brick5'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='36'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data5/lv_hdfs5'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='45'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data6/lv_brick6'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='38'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data6/lv_hdfs6'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='46'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data7/lv_brick7'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='40'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='3.62t'|LVM2_PV_NAME='/dev/data7/lv_hdfs7'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='47'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='100.00g'|LVM2_PV_NAME='/dev/loop0'|LVM2_PV_MAJOR='7'|LVM2_PV_MINOR='0'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='2.00g'|LVM2_PV_NAME='/dev/loop1'|LVM2_PV_MAJOR='7'|LVM2_http://m.abc.com/obituaries/2017/apr/10/charles-dietz-2017-04-10/PV_MINOR='1'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='100.00g'|LVM2_PV_NAME='/dev/mapper/docker-253:3-100685290-pool'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='5'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='976.00g'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/home'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='2'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='2.57t'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/opt'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='4'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='50.00g'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/root'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='0'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='16.00g'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/swap'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='1'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='50.00g'|LVM2_PV_NAME='/dev/rhel_ceehadoop1/var'|LVM2_PV_MAJOR='253'|LVM2_PV_MINOR='3'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT=''|LVM2_PV_UUID=''|LVM2_DEV_SIZE='1.00g'|LVM2_PV_NAME='/dev/sda2'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='2'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='0 '|LVM2_PV_EXT_VSN=''|LVM2_PE_START='0 '|LVM2_PV_SIZE='0 '|LVM2_PV_FREE='0 '|LVM2_PV_USED='0 '|LVM2_PV_ATTR='---'|LVM2_PV_ALLOCATABLE=''|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='0'|LVM2_PV_PE_ALLOC_COUNT='0'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='0'|LVM2_PV_MDA_USED_COUNT='0'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE=''|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME=''
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='3e2mRe-1y3a-iUSj-F3tH-yG1M-t3qD-8uNDf5'|LVM2_DEV_SIZE='3.63t'|LVM2_PV_NAME='/dev/sda3'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='3'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='1020.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='1.00m'|LVM2_PV_SIZE='3.63t'|LVM2_PV_FREE='0 '|LVM2_PV_USED='3.63t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='952831'|LVM2_PV_PE_ALLOC_COUNT='952831'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='rhel_ceehadoop1'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='dhr134-VxSA-R7dW-op7s-HCco-7J2A-7MvaeN'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdb1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='17'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data1'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='sxTsQE-raAM-8s3b-FSy0-LhL2-Naue-f2ebcA'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdc1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='33'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data2'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='yxj2qs-qOKA-57XF-228r-qWQj-3zez-i0Vaiw'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdd1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='49'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data3'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='94oczR-Utni-dOzj-3CMQ-efOJ-TVa8-yak25P'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sde1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='65'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data4'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='r3UW8H-pLz5-rpsn-GK2u-1kJE-GKsy-uOeoIz'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdf1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='81'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data5'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='oZtJSv-HM9w-4She-1TU0-Z31G-dgKX-liJNID'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdg1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='97'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data6'
  LVM2_PV_FMT='lvm2'|LVM2_PV_UUID='Ys0p6S-WJOW-TAZT-czmX-iOYq-0Tgg-v4cl0H'|LVM2_DEV_SIZE='3.64t'|LVM2_PV_NAME='/dev/sdh1'|LVM2_PV_MAJOR='8'|LVM2_PV_MINOR='113'|LVM2_PV_MDA_FREE='0 '|LVM2_PV_MDA_SIZE='252.00k'|LVM2_PV_EXT_VSN='2'|LVM2_PE_START='256.00k'|LVM2_PV_SIZE='3.64t'|LVM2_PV_FREE='4.00m'|LVM2_PV_USED='3.64t'|LVM2_PV_ATTR='a--'|LVM2_PV_ALLOCATABLE='allocatable'|LVM2_PV_EXPORTED=''|LVM2_PV_MISSING=''|LVM2_PV_PE_COUNT='953853'|LVM2_PV_PE_ALLOC_COUNT='953852'|LVM2_PV_TAGS=''|LVM2_PV_MDA_COUNT='1'|LVM2_PV_MDA_USED_COUNT='1'|LVM2_PV_BA_START='0 '|LVM2_PV_BA_SIZE='0 '|LVM2_PV_IN_USE='used'|LVM2_PV_DUPLICATE=''|LVM2_VG_NAME='data7'
""".strip()   # noqa: W291

VGS_HEADINGS = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
  VG              Attr   Ext   #PV #LV #SN VSize VFree VG UUID                                VProfile #VMda VMdaFree  VMdaSize  #VMdaUse VG Tags
  data1           wz--n- 4.00m   1   3   0 3.64t 4.00m cYvfbB-tIzb-d4yt-5AFp-wM66-IBRx-JxF0zp              1        0    252.00k        1        
  data2           wz--n- 4.00m   1   3   0 3.64t 4.00m kdziV0-Xoh0-9al9-5E9N-1WU3-qCoH-v96zbG              1        0    252.00k        1        
  data3           wz--n- 4.00m   1   3   0 3.64t 4.00m rZUP2i-zoTb-HbrL-Ehk1-6kEr-Km20-UrbZfn              1        0    252.00k        1        
  data4           wz--n- 4.00m   1   3   0 3.64t 4.00m 6eCaHM-Yaoj-A2vy-V12P-mA0x-l6Hf-WwTlz0              1        0    252.00k        1        
  data5           wz--n- 4.00m   1   3   0 3.64t 4.00m tcu5Ur-WtaF-0fES-LEQE-YzVi-WYYU-iTyEDT              1        0    252.00k        1        
  data6           wz--n- 4.00m   1   3   0 3.64t 4.00m ey8e3g-BkbG-u877-TVQW-S7df-hsF9-D6BkCe              1        0    252.00k        1        
  data7           wz--n- 4.00m   1   3   0 3.64t 4.00m r58iLs-sYla-lC4h-UURW-bZOG-P2Yd-lambpM              1        0    252.00k        1        
  rhel_ceehadoop1 wz--n- 4.00m   1   5   0 3.63t    0  IIfj5y-6deS-aBaO-M4no-sQUg-0UEU-wo0IFM              1        0   1020.00k        1        
    Reloading config files
    Wiping internal VG cache
""".strip()   # noqa: W291

VGS_NO_HEADINGS = """
  WARNING: Locking disabled. Be careful! This could corrupt your metadata.
  LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='cYvfbB-tIzb-d4yt-5AFp-wM66-IBRx-JxF0zp'|LVM2_VG_NAME='data1'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='3.64t'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCK_TYPE=''|LVM2_VG_LOCK_ARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='953853'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_VG_MISSING_PV_COUNT='0'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='8'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='252.00k'|LVM2_VG_MDA_COPIES='unmanaged'
  LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='kdziV0-Xoh0-9al9-5E9N-1WU3-qCoH-v96zbG'|LVM2_VG_NAME='data2'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='3.64t'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCK_TYPE=''|LVM2_VG_LOCK_ARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='953853'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_VG_MISSING_PV_COUNT='0'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='8'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='252.00k'|LVM2_VG_MDA_COPIES='unmanaged'
  LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='rZUP2i-zoTb-HbrL-Ehk1-6kEr-Km20-UrbZfn'|LVM2_VG_NAME='data3'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='3.64t'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCK_TYPE=''|LVM2_VG_LOCK_ARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='953853'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_VG_MISSING_PV_COUNT='0'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='8'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='252.00k'|LVM2_VG_MDA_COPIES='unmanaged'
  LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='6eCaHM-Yaoj-A2vy-V12P-mA0x-l6Hf-WwTlz0'|LVM2_VG_NAME='data4'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='3.64t'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCK_TYPE=''|LVM2_VG_LOCK_ARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='953853'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_VG_MISSING_PV_COUNT='0'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='8'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='252.00k'|LVM2_VG_MDA_COPIES='unmanaged'
  LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='tcu5Ur-WtaF-0fES-LEQE-YzVi-WYYU-iTyEDT'|LVM2_VG_NAME='data5'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='3.64t'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCK_TYPE=''|LVM2_VG_LOCK_ARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='953853'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_VG_MISSING_PV_COUNT='0'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='8'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='252.00k'|LVM2_VG_MDA_COPIES='unmanaged'
  LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='ey8e3g-BkbG-u877-TVQW-S7df-hsF9-D6BkCe'|LVM2_VG_NAME='data6'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='3.64t'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCK_TYPE=''|LVM2_VG_LOCK_ARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='953853'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_VG_MISSING_PV_COUNT='0'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='8'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='252.00k'|LVM2_VG_MDA_COPIES='unmanaged'
  LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='r58iLs-sYla-lC4h-UURW-bZOG-P2Yd-lambpM'|LVM2_VG_NAME='data7'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='3.64t'|LVM2_VG_FREE='4.00m'|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCK_TYPE=''|LVM2_VG_LOCK_ARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='953853'|LVM2_VG_FREE_COUNT='1'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_VG_MISSING_PV_COUNT='0'|LVM2_LV_COUNT='3'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='8'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='252.00k'|LVM2_VG_MDA_COPIES='unmanaged'
  LVM2_VG_FMT='lvm2'|LVM2_VG_UUID='IIfj5y-6deS-aBaO-M4no-sQUg-0UEU-wo0IFM'|LVM2_VG_NAME='rhel_ceehadoop1'|LVM2_VG_ATTR='wz--n-'|LVM2_VG_PERMISSIONS='writeable'|LVM2_VG_EXTENDABLE='extendable'|LVM2_VG_EXPORTED=''|LVM2_VG_PARTIAL=''|LVM2_VG_ALLOCATION_POLICY='normal'|LVM2_VG_CLUSTERED=''|LVM2_VG_SIZE='3.63t'|LVM2_VG_FREE='0 '|LVM2_VG_SYSID=''|LVM2_VG_SYSTEMID=''|LVM2_VG_LOCK_TYPE=''|LVM2_VG_LOCK_ARGS=''|LVM2_VG_EXTENT_SIZE='4.00m'|LVM2_VG_EXTENT_COUNT='952831'|LVM2_VG_FREE_COUNT='0'|LVM2_MAX_LV='0'|LVM2_MAX_PV='0'|LVM2_PV_COUNT='1'|LVM2_VG_MISSING_PV_COUNT='0'|LVM2_LV_COUNT='5'|LVM2_SNAP_COUNT='0'|LVM2_VG_SEQNO='6'|LVM2_VG_TAGS=''|LVM2_VG_PROFILE=''|LVM2_VG_MDA_COUNT='1'|LVM2_VG_MDA_USED_COUNT='1'|LVM2_VG_MDA_FREE='0 '|LVM2_VG_MDA_SIZE='1020.00k'|LVM2_VG_MDA_COPIES='unmanaged'
""".strip()   # noqa: W291

PRIMARY_DATA = [
    {'a': 1, 'b': 2, 'c': 3, 'd': 4, 'name_key': 'xyz'},
    {'a': None, 'b': 12, 'c': 13, 'd': 14, 'name_key': 'qrs'},
    {'a': None, 'b': 12, 'c': 13, 'd': 14, 'f': '', 'name_key': 'def'}
]

PRIMARY_KEY_DATA = {
    'xyz': {'a': 1, 'b': 2, 'c': 3, 'd': 4, 'name_key': 'xyz'},
    'qrs': {'a': None, 'b': 12, 'c': 13, 'd': 14, 'name_key': 'qrs'},
    'def': {'a': None, 'b': 12, 'c': 13, 'd': 14, 'f': '', 'name_key': 'def'}
}

SECONDARY_DATA = [
    {'a': 31, 'e': 33, 'name_key': 'xyz'},
    {'a': 11, 'e': 23, 'name_key': 'qrs'},
    {'a': 1, 'e': 3, 'name_key': 'ghi'}
]


class DataClass:
    def __init__(self):
        self.data = [PRIMARY_DATA, SECONDARY_DATA]


class ContentClass:
    def __init__(self):
        self.data = {'content': [PRIMARY_DATA, SECONDARY_DATA]}


def test_get_shared_data():
    data = lvm.get_shared_data(DataClass())
    assert data == [PRIMARY_DATA, SECONDARY_DATA]
    data = lvm.get_shared_data(ContentClass())
    assert data == [PRIMARY_DATA, SECONDARY_DATA]


def test_to_name_key_dict():
    assert lvm.to_name_key_dict(PRIMARY_DATA, 'name_key') == PRIMARY_KEY_DATA


def test_merge_lvm_data():
    combined = lvm.merge_lvm_data(PRIMARY_DATA, SECONDARY_DATA, 'name_key')
    assert combined == {
        'xyz': {'a': 1, 'b': 2, 'c': 3, 'd': 4, 'e': 33, 'name_key': 'xyz'},
        'qrs': {'a': 11, 'b': 12, 'c': 13, 'd': 14, 'e': 23, 'name_key': 'qrs'},
        'def': {'a': None, 'b': 12, 'c': 13, 'd': 14, 'f': None, 'name_key': 'def'},
        'ghi': {'a': 1, 'e': 3, 'name_key': 'ghi'}
    }


def test_set_defaults():
    assert lvm.set_defaults(PRIMARY_KEY_DATA) == {
        'xyz': {'a': 1, 'b': 2, 'c': 3, 'd': 4, 'name_key': 'xyz'},
        'qrs': {'a': None, 'b': 12, 'c': 13, 'd': 14, 'name_key': 'qrs'},
        'def': {'a': None, 'b': 12, 'c': 13, 'd': 14, 'f': None, 'name_key': 'def'}
    }


@pytest.fixture
def lvm_data():
    vgs = Vgs(context_wrap(VGS_NO_HEADINGS))
    vgs_headings = VgsHeadings(context_wrap(VGS_HEADINGS))
    pvs = Pvs(context_wrap(PVS_NO_HEADINGS))
    pvs_headings = PvsHeadings(context_wrap(PVS_HEADINGS))
    lvs = Lvs(context_wrap(LVS_NO_HEADINGS))
    lvs_headings = LvsHeadings(context_wrap(LVS_HEADINGS))
    shared_list = [
        {Vgs: vgs, VgsHeadings: vgs_headings},
        {Vgs: vgs},
        {VgsHeadings: vgs_headings},
        {Pvs: pvs, PvsHeadings: pvs_headings},
        {Pvs: pvs},
        {PvsHeadings: pvs_headings},
        {Lvs: lvs, LvsHeadings: lvs_headings},
        {Lvs: lvs},
        {LvsHeadings: lvs_headings}
    ]
    LvmData = namedtuple('LvmData', ['lvm_info', 'shared'])
    yield [LvmData(Lvm(shared.get(Lvs),
                       shared.get(LvsHeadings),
                       shared.get(Pvs),
                       shared.get(PvsHeadings),
                       shared.get(Vgs),
                       shared.get(VgsHeadings)),
                   shared)
           for shared in shared_list]


def test_combiner_vgs(lvm_data):
    for data in lvm_data:
        if Vgs in data.shared or VgsHeadings in data.shared:
            lvm_info = data.lvm_info
            assert lvm_info.volume_groups is not None
            assert len(list(lvm_info.volume_groups)) == 8
            assert lvm_info.volume_groups['data7']['VG_UUID'] == 'r58iLs-sYla-lC4h-UURW-bZOG-P2Yd-lambpM'
            assert lvm_info.volume_group_names == set([
                'data1', 'data2', 'data3', 'data4', 'data5', 'data6', 'data7',
                'rhel_ceehadoop1'
            ])
            if Vgs in data.shared:
                assert lvm_info.volume_groups['data1']['VPerms'] == 'writeable'
            else:
                assert lvm_info.volume_groups['data1']['VPerms'] is None
            filtered_lvm_info = lvm_info.filter_volume_groups('data')
            assert set(filtered_lvm_info.keys()) == set([
                'data1', 'data2', 'data3', 'data4', 'data5', 'data6', 'data7'
            ])
            assert filtered_lvm_info['data7']['VG_UUID'] == 'r58iLs-sYla-lC4h-UURW-bZOG-P2Yd-lambpM'


def test_combiner_pvs(lvm_data):
    for data in lvm_data:
        if Pvs in data.shared or PvsHeadings in data.shared:
            lvm_info = data.lvm_info
            assert lvm_info.physical_volumes is not None
            assert len(list(lvm_info.physical_volumes)) == 31
            assert lvm_info.physical_volumes['/dev/sdh1+Ys0p6S-WJOW-TAZT-czmX-iOYq-0Tgg-v4cl0H']['VG'] == 'data7'
            assert lvm_info.physical_volume_names == set([
                '/dev/data1/lv_brick1',
                '/dev/data1/lv_hdfs1',
                '/dev/data2/lv_brick2',
                '/dev/data2/lv_hdfs2',
                '/dev/data3/lv_brick3',
                '/dev/data3/lv_hdfs3',
                '/dev/data4/lv_brick4',
                '/dev/data4/lv_hdfs4',
                '/dev/data5/lv_brick5',
                '/dev/data5/lv_hdfs5',
                '/dev/data6/lv_brick6',
                '/dev/data6/lv_hdfs6',
                '/dev/data7/lv_brick7',
                '/dev/data7/lv_hdfs7',
                '/dev/loop0',
                '/dev/loop1',
                '/dev/mapper/docker-253:3-100685290-pool',
                '/dev/rhel_ceehadoop1/home',
                '/dev/rhel_ceehadoop1/opt',
                '/dev/rhel_ceehadoop1/root',
                '/dev/rhel_ceehadoop1/swap',
                '/dev/rhel_ceehadoop1/var',
                '/dev/sda2',
                '/dev/sda3',
                '/dev/sdb1',
                '/dev/sdc1',
                '/dev/sdd1',
                '/dev/sde1',
                '/dev/sdf1',
                '/dev/sdg1',
                '/dev/sdh1'
            ])
            pv_sdg1 = lvm_info.filter_physical_volumes('/dev/sdg1')
            assert len(pv_sdg1) == 1
            pv_sdg1_key, pv_sdg1_values = pv_sdg1.popitem()
            assert '/dev/sdg1' in lvm_info.physical_volume_names
            if Pvs in data.shared:
                assert lvm_info.physical_volumes[pv_sdg1_key]['LVM2_PV_MINOR'] == '97'
                assert lvm_info.physical_volumes[pv_sdg1_key]['Missing'] is None
            else:
                assert 'LVM2_PV_MINOR' not in lvm_info.physical_volumes[pv_sdg1_key]
                assert lvm_info.physical_volumes[pv_sdg1_key]['Missing'] is None
            filtered_lvm_info = lvm_info.filter_physical_volumes('sda')
            assert set([p['PV'] for p in filtered_lvm_info.values()]) == set(['/dev/sda2', '/dev/sda3'])
            for pv in filtered_lvm_info.values():
                if pv['PV'] == '/dev/sda2':
                    assert pv['DevSize'] == '1.00g'


def test_combiner_lvs(lvm_data):
    for data in lvm_data:
        if Lvs in data.shared or LvsHeadings in data.shared:
            lvm_info = data.lvm_info
            assert lvm_info.logical_volumes is not None
            assert len(list(lvm_info.logical_volumes)) == 40
            pool7 = lvm_info.logical_volumes[Lvm.LvVgName(LV='pool7', VG='data7')]
            if Lvs in data.shared:
                assert pool7['LVM2_REGION_SIZE'] == '0'
                for k, v in LVS_NOHEADINGS_POOL7.items():
                    assert pool7[k] == v
            else:
                assert 'LVM2_REGION_SIZE' not in pool7
                assert pool7['Region'] is None
            if LvsHeadings in data.shared:
                for k, v in LVS_HEADINGS_POOL7.items():
                    assert pool7[k] == v
            assert lvm_info.logical_volume_names == set([
                Lvm.LvVgName(LV='lv_brick1', VG='data1'),
                Lvm.LvVgName(LV='lv_hdfs1', VG='data1'),
                Lvm.LvVgName(LV='pool1', VG='data1'),
                Lvm.LvVgName(LV='[pool1_tdata]', VG='data1'),
                Lvm.LvVgName(LV='[pool1_tmeta]', VG='data1'),
                Lvm.LvVgName(LV='lv_brick2', VG='data2'),
                Lvm.LvVgName(LV='lv_hdfs2', VG='data2'),
                Lvm.LvVgName(LV='pool2', VG='data2'),
                Lvm.LvVgName(LV='[pool2_tdata]', VG='data2'),
                Lvm.LvVgName(LV='[pool2_tmeta]', VG='data2'),
                Lvm.LvVgName(LV='lv_brick3', VG='data3'),
                Lvm.LvVgName(LV='lv_hdfs3', VG='data3'),
                Lvm.LvVgName(LV='pool3', VG='data3'),
                Lvm.LvVgName(LV='[pool3_tdata]', VG='data3'),
                Lvm.LvVgName(LV='[pool3_tmeta]', VG='data3'),
                Lvm.LvVgName(LV='lv_brick4', VG='data4'),
                Lvm.LvVgName(LV='lv_hdfs4', VG='data4'),
                Lvm.LvVgName(LV='pool4', VG='data4'),
                Lvm.LvVgName(LV='[pool4_tdata]', VG='data4'),
                Lvm.LvVgName(LV='[pool4_tmeta]', VG='data4'),
                Lvm.LvVgName(LV='lv_brick5', VG='data5'),
                Lvm.LvVgName(LV='lv_hdfs5', VG='data5'),
                Lvm.LvVgName(LV='pool5', VG='data5'),
                Lvm.LvVgName(LV='[pool5_tdata]', VG='data5'),
                Lvm.LvVgName(LV='[pool5_tmeta]', VG='data5'),
                Lvm.LvVgName(LV='lv_brick6', VG='data6'),
                Lvm.LvVgName(LV='lv_hdfs6', VG='data6'),
                Lvm.LvVgName(LV='pool6', VG='data6'),
                Lvm.LvVgName(LV='[pool6_tdata]', VG='data6'),
                Lvm.LvVgName(LV='[pool6_tmeta]', VG='data6'),
                Lvm.LvVgName(LV='lv_brick7', VG='data7'),
                Lvm.LvVgName(LV='lv_hdfs7', VG='data7'),
                Lvm.LvVgName(LV='pool7', VG='data7'),
                Lvm.LvVgName(LV='[pool7_tdata]', VG='data7'),
                Lvm.LvVgName(LV='[pool7_tmeta]', VG='data7'),
                Lvm.LvVgName(LV='home', VG='rhel_ceehadoop1'),
                Lvm.LvVgName(LV='opt', VG='rhel_ceehadoop1'),
                Lvm.LvVgName(LV='root', VG='rhel_ceehadoop1'),
                Lvm.LvVgName(LV='swap', VG='rhel_ceehadoop1'),
                Lvm.LvVgName(LV='var', VG='rhel_ceehadoop1'),
            ])
            filtered_lvm_info = lvm_info.filter_logical_volumes('pool7')
            for lv in filtered_lvm_info:
                assert lv.LV in ['pool7', '[pool7_tdata]', '[pool7_tmeta]']
            assert Lvm.LvVgName(LV='pool7', VG='data7') in filtered_lvm_info


@pytest.fixture
def lvm_data_all():
    vgsall = VgsAll(context_wrap(VGS_NO_HEADINGS))
    pvsall = PvsAll(context_wrap(PVS_NO_HEADINGS))
    lvsall = LvsAll(context_wrap(LVS_NO_HEADINGS))
    shared_list = [
        {VgsAll: vgsall},
        {PvsAll: pvsall},
        {LvsAll: lvsall},
    ]
    LvmData = namedtuple('LvmData', ['lvm_info', 'shared'])
    yield [LvmData(LvmAll(shared.get(LvsAll),
                          shared.get(PvsAll),
                          shared.get(VgsAll)),
                   shared)
           for shared in shared_list]


def test_combiner_vgsall(lvm_data_all):
    has_vgsall = False
    for data in lvm_data_all:
        if VgsAll in data.shared:
            has_vgsall = True
            lvm_info = data.lvm_info
            assert lvm_info.volume_groups is not None
            assert len(list(lvm_info.volume_groups)) == 8
            assert lvm_info.volume_groups['data7']['VG_UUID'] == 'r58iLs-sYla-lC4h-UURW-bZOG-P2Yd-lambpM'
            assert lvm_info.volume_group_names == set([
                'data1', 'data2', 'data3', 'data4', 'data5', 'data6', 'data7',
                'rhel_ceehadoop1'
            ])
            assert lvm_info.volume_groups['data1']['VPerms'] == 'writeable'
            filtered_lvm_info = lvm_info.filter_volume_groups('data')
            assert set(filtered_lvm_info.keys()) == set([
                'data1', 'data2', 'data3', 'data4', 'data5', 'data6', 'data7'
            ])
            assert filtered_lvm_info['data7']['VG_UUID'] == 'r58iLs-sYla-lC4h-UURW-bZOG-P2Yd-lambpM'
    assert has_vgsall


def test_combiner_pvsall(lvm_data_all):
    has_pvsall = False
    for data in lvm_data_all:
        if PvsAll in data.shared:
            has_pvsall = True
            lvm_info = data.lvm_info
            assert lvm_info.physical_volumes is not None
            assert len(list(lvm_info.physical_volumes)) == 31
            assert lvm_info.physical_volumes['/dev/sdh1+Ys0p6S-WJOW-TAZT-czmX-iOYq-0Tgg-v4cl0H']['VG'] == 'data7'
            assert lvm_info.physical_volume_names == set([
                '/dev/data1/lv_brick1',
                '/dev/data1/lv_hdfs1',
                '/dev/data2/lv_brick2',
                '/dev/data2/lv_hdfs2',
                '/dev/data3/lv_brick3',
                '/dev/data3/lv_hdfs3',
                '/dev/data4/lv_brick4',
                '/dev/data4/lv_hdfs4',
                '/dev/data5/lv_brick5',
                '/dev/data5/lv_hdfs5',
                '/dev/data6/lv_brick6',
                '/dev/data6/lv_hdfs6',
                '/dev/data7/lv_brick7',
                '/dev/data7/lv_hdfs7',
                '/dev/loop0',
                '/dev/loop1',
                '/dev/mapper/docker-253:3-100685290-pool',
                '/dev/rhel_ceehadoop1/home',
                '/dev/rhel_ceehadoop1/opt',
                '/dev/rhel_ceehadoop1/root',
                '/dev/rhel_ceehadoop1/swap',
                '/dev/rhel_ceehadoop1/var',
                '/dev/sda2',
                '/dev/sda3',
                '/dev/sdb1',
                '/dev/sdc1',
                '/dev/sdd1',
                '/dev/sde1',
                '/dev/sdf1',
                '/dev/sdg1',
                '/dev/sdh1'
            ])
            pv_sdg1 = lvm_info.filter_physical_volumes('/dev/sdg1')
            assert len(pv_sdg1) == 1
            pv_sdg1_key, pv_sdg1_values = pv_sdg1.popitem()
            assert '/dev/sdg1' in lvm_info.physical_volume_names
            assert lvm_info.physical_volumes[pv_sdg1_key]['LVM2_PV_MINOR'] == '97'
            assert lvm_info.physical_volumes[pv_sdg1_key]['Missing'] is None
            filtered_lvm_info = lvm_info.filter_physical_volumes('sda')
            assert set([p['PV'] for p in filtered_lvm_info.values()]) == set(['/dev/sda2', '/dev/sda3'])
            for pv in filtered_lvm_info.values():
                if pv['PV'] == '/dev/sda2':
                    assert pv['DevSize'] == '1.00g'
    assert has_pvsall


def test_combiner_lvsall(lvm_data_all):
    has_lvsall = False
    for data in lvm_data_all:
        if LvsAll in data.shared:
            has_lvsall = True
            lvm_info = data.lvm_info
            assert lvm_info.logical_volumes is not None
            assert len(list(lvm_info.logical_volumes)) == 40
            pool7 = lvm_info.logical_volumes[Lvm.LvVgName(LV='pool7', VG='data7')]
            assert pool7['LVM2_REGION_SIZE'] == '0'
            for k in LVS_NOHEADINGS_POOL7.keys():
                assert pool7[k] == LVS_NOHEADINGS_POOL7[k]
            assert lvm_info.logical_volume_names == set([
                LvmAll.LvVgName(LV='lv_brick1', VG='data1'),
                LvmAll.LvVgName(LV='lv_hdfs1', VG='data1'),
                LvmAll.LvVgName(LV='pool1', VG='data1'),
                LvmAll.LvVgName(LV='[pool1_tdata]', VG='data1'),
                LvmAll.LvVgName(LV='[pool1_tmeta]', VG='data1'),
                LvmAll.LvVgName(LV='lv_brick2', VG='data2'),
                LvmAll.LvVgName(LV='lv_hdfs2', VG='data2'),
                LvmAll.LvVgName(LV='pool2', VG='data2'),
                LvmAll.LvVgName(LV='[pool2_tdata]', VG='data2'),
                LvmAll.LvVgName(LV='[pool2_tmeta]', VG='data2'),
                LvmAll.LvVgName(LV='lv_brick3', VG='data3'),
                LvmAll.LvVgName(LV='lv_hdfs3', VG='data3'),
                LvmAll.LvVgName(LV='pool3', VG='data3'),
                LvmAll.LvVgName(LV='[pool3_tdata]', VG='data3'),
                LvmAll.LvVgName(LV='[pool3_tmeta]', VG='data3'),
                LvmAll.LvVgName(LV='lv_brick4', VG='data4'),
                LvmAll.LvVgName(LV='lv_hdfs4', VG='data4'),
                LvmAll.LvVgName(LV='pool4', VG='data4'),
                LvmAll.LvVgName(LV='[pool4_tdata]', VG='data4'),
                LvmAll.LvVgName(LV='[pool4_tmeta]', VG='data4'),
                LvmAll.LvVgName(LV='lv_brick5', VG='data5'),
                LvmAll.LvVgName(LV='lv_hdfs5', VG='data5'),
                LvmAll.LvVgName(LV='pool5', VG='data5'),
                LvmAll.LvVgName(LV='[pool5_tdata]', VG='data5'),
                LvmAll.LvVgName(LV='[pool5_tmeta]', VG='data5'),
                LvmAll.LvVgName(LV='lv_brick6', VG='data6'),
                LvmAll.LvVgName(LV='lv_hdfs6', VG='data6'),
                LvmAll.LvVgName(LV='pool6', VG='data6'),
                LvmAll.LvVgName(LV='[pool6_tdata]', VG='data6'),
                LvmAll.LvVgName(LV='[pool6_tmeta]', VG='data6'),
                LvmAll.LvVgName(LV='lv_brick7', VG='data7'),
                LvmAll.LvVgName(LV='lv_hdfs7', VG='data7'),
                LvmAll.LvVgName(LV='pool7', VG='data7'),
                LvmAll.LvVgName(LV='[pool7_tdata]', VG='data7'),
                LvmAll.LvVgName(LV='[pool7_tmeta]', VG='data7'),
                LvmAll.LvVgName(LV='home', VG='rhel_ceehadoop1'),
                LvmAll.LvVgName(LV='opt', VG='rhel_ceehadoop1'),
                LvmAll.LvVgName(LV='root', VG='rhel_ceehadoop1'),
                LvmAll.LvVgName(LV='swap', VG='rhel_ceehadoop1'),
                LvmAll.LvVgName(LV='var', VG='rhel_ceehadoop1'),
            ])
            filtered_lvm_info = lvm_info.filter_logical_volumes('pool7')
            for lv in filtered_lvm_info:
                assert lv.LV in ['pool7', '[pool7_tdata]', '[pool7_tmeta]']
            assert LvmAll.LvVgName(LV='pool7', VG='data7') in filtered_lvm_info
    assert has_lvsall
